@@ -1,4 +1,4 @@
-# last modified 21 August 2009 by J. Fox
+# last modified 25 March 2011 by J. Fox
 
 # Data menu dialogs
 
@@ -326,7 +326,7 @@ readDataSet <- function() {
 		location <- tclvalue(locationVariable)
 		file <- if (location == "clipboard") "clipboard" 
 			else if (location == "local") tclvalue(tkgetOpenFile(filetypes=
-							gettextRcmdr('{"Text Files" {".txt" ".TXT" ".dat" ".DAT" ".csv" ".CSV"}} {"All Files" {"*"}}')))
+							gettextRcmdr('{"All Files" {"*"}} {"Text Files" {".txt" ".TXT" ".dat" ".DAT" ".csv" ".CSV"}}')))
 			else {
 				initializeDialog(subdialog, title=gettextRcmdr("Internet URL"))
 				onOKsub <- function(){
@@ -386,6 +386,7 @@ readDataSet <- function() {
 
 readDataFromPackage <- function() {
 	env <- environment()
+	datasets <- NULL
 	initializeDialog(title=gettextRcmdr("Read Data From Package"))
 	dsname <- tclVar("")
 	package <- NULL
@@ -393,7 +394,13 @@ readDataFromPackage <- function() {
 	entryDsname <- ttkentry(enterFrame, width="20", textvariable=dsname)
 	packages <- sort(.packages())
 	packages <- packages[! packages %in% c("base", "stats")]
-	packages <- packages[sapply(packages, function(package) nrow(data(package=package)$results) > 0)]
+	packages <- packages[sapply(packages, function(package){
+				ds <- data(package=package)$results
+				if (nrow(ds) == 0) return(FALSE)
+				ds <- ds[, "Item"]
+				valid <- sapply(ds, is.valid.name)
+				length(ds[valid]) > 0
+			})]
 	packageDatasetFrame <- tkframe(top)
 	packageFrame <- tkframe(packageDatasetFrame)
 	packageBox <- tklistbox(packageFrame, height="4", exportselection="FALSE",
@@ -410,7 +417,9 @@ readDataFromPackage <- function() {
 	tkconfigure(datasetBox, yscrollcommand=function(...) tkset(datasetScroll, ...))
 	onPackageSelect <- function(){
 		assign("package", packages[as.numeric(tkcurselection(packageBox)) + 1], envir=env)
-		datasets <- data(package=package)$results[,3]
+		datasets <<- data(package=package)$results[,3]
+		valid <- sapply(datasets, is.valid.name)
+		datasets <<- datasets[valid]
 		tkdelete(datasetBox, "0", "end")
 		for (dataset in datasets) tkinsert(datasetBox, "end", dataset)
 		tkconfigure(datasetBox, height=min(4, length(datasets)))
@@ -457,7 +466,7 @@ readDataFromPackage <- function() {
 		tkbind(datasetBox, "<ButtonPress-1>", onClick)
 	}
 	onDatasetSelect <- function(){
-		tclvalue(dsname) <- data(package=package)$results[as.numeric(tkcurselection(datasetBox)) + 1,3]
+		tclvalue(dsname) <- datasets[as.numeric(tkcurselection(datasetBox)) + 1]
 	}
 	firstChar <- tolower(substr(packages, 1, 1))
 	len <- length(packages)
@@ -501,7 +510,7 @@ readDataFromPackage <- function() {
 	onClick <- function() tkfocus(packageBox)
 	tkbind(packageBox, "<ButtonPress-1>", onClick)
 	onOK <- function(){
-		datasetName <- data(package=package)$results[as.numeric(tkcurselection(datasetBox)) + 1,3]
+		datasetName <- datasets[as.numeric(tkcurselection(datasetBox)) + 1]
 		dsnameValue <- tclvalue(dsname)
 		if (dsnameValue != "" && is.null(package)){
 			closeDialog()
@@ -608,7 +617,7 @@ importSPSS <- function() {
 			}
 		}
 		file <- tclvalue(tkgetOpenFile(
-				filetypes=gettextRcmdr('{"SPSS save files" {".sav" ".SAV"}} {"SPSS portable files" {".por" ".POR"}} {"All Files" {"*"}}')))
+				filetypes=gettextRcmdr('{"All Files" {"*"}} {"SPSS portable files" {".por" ".POR"}} {"SPSS save files" {".sav" ".SAV"}}')))
 		if (file == "") {
 			tkfocus(CommanderWindow())
 			return()
@@ -663,7 +672,7 @@ importMinitab <- function() {
 			}
 		}
 		file <- tclvalue(tkgetOpenFile(
-				filetypes=gettextRcmdr('{"Minitab portable files" {".mtp" ".MTP"}} {"All Files" {"*"}}')))
+				filetypes=gettextRcmdr('{"All Files" {"*"}} {"Minitab portable files" {".mtp" ".MTP"}}')))
 		if (file == "") {
 			tkfocus(CommanderWindow())
 			return()
@@ -730,7 +739,7 @@ importSTATA <- function() {
 			}
 		}
 		file <- tclvalue(tkgetOpenFile(
-				filetypes=gettextRcmdr('{"STATA datasets" {".dta" ".DTA"}} {"All Files" {"*"}}')))
+				filetypes=gettextRcmdr('{"All Files" {"*"}} {"STATA datasets" {".dta" ".DTA"}}')))
 		if (file == "") {
 			tkfocus(CommanderWindow())
 			return()
@@ -806,7 +815,7 @@ importRODBCtable <- function(){
 			}
 		}
 		File <- tclvalue(tkgetOpenFile(filetypes = gettextRcmdr(
-					'{"MS Excel file" {*.xls ".XLS"}} {"MS Excel 2007 file" {*.xlsx ".XLSX"}} {"MS Access database" {*.mdb ".MDB"}} {"MS Access 2007 database" {*.accdb ".ACCDB"}} {"dBase-like file" {*.dbf ".DBF"}} {"All Files" {"*"}}'
+					'{"All Files" {"*"}} {"MS Access database" {".mdb" ".MDB"}} {"MS Access 2007 database" {".accdb" ".ACCDB"}} {"dBase-like file" {".dbf" ".DBF"}} {"MS Excel 2007 file" {".xlsx" ".XLSX"}} {"MS Excel file" {".xls" ".XLS"}}'
 				)))
 		if(File == ""){
 			tkfocus(CommanderWindow())
@@ -1259,7 +1268,7 @@ exportDataSet <- function() {
 			else if (delim == "spaces") " "
 			else if (delim == "commas") ","
 			else trim.blanks(tclvalue(otherVariable))
-		saveFile <- tclvalue(tkgetSaveFile(filetypes=gettextRcmdr('{"Text Files" {".txt" ".TXT" ".dat" ".DAT" ".csv" ".CSV"}} {"All Files" {"*"}}'),
+		saveFile <- tclvalue(tkgetSaveFile(filetypes=gettextRcmdr('{"All Files" {"*"}} {"Text Files" {".txt" ".TXT" ".dat" ".DAT" ".csv" ".CSV"}}'),
 				defaultextension="txt", initialfile=paste(dsname, ".txt", sep="")))
 		if (saveFile == "") {
 			tkfocus(CommanderWindow())
@@ -1716,7 +1725,7 @@ Stack <- function(){
 
 loadDataSet <- function() {
 	file <- tclvalue(tkgetOpenFile(filetypes=
-				gettextRcmdr('{"R Data Files" {".rda" ".Rda" ".RDA" ".RData"}} {"All Files" {"*"}}')))
+				gettextRcmdr('{"All Files" {"*"}} {"R Data Files" {".RData" ".rda" ".Rda" ".RDA"}}')))
 	if (file == "") return()
 	command <- paste('load("', file,'")', sep="")
 	dsname <- justDoIt(command)
@@ -1727,8 +1736,8 @@ loadDataSet <- function() {
 
 saveDataSet <- function() {
 	file <- tclvalue(tkgetSaveFile(filetypes=
-				gettextRcmdr('{"R Data Files" {".rda" ".Rda" ".RDA" ".RData"}} {"All Files" {"*"}}'),
-			defaultextension="rda", initialfile=paste(activeDataSet(), "rda", sep=".")))
+				gettextRcmdr('{"All Files" {"*"}} {"R Data Files" {".RData" ".rda" ".Rda" ".RDA"}}'),
+			defaultextension=".RData", initialfile=paste(activeDataSet(), ".RData", sep="")))
 	if (file == "") return()
 	command <- paste('save("', activeDataSet(), '", file="', file, '")', sep="")
 	justDoIt(command)
